import json
from enum import Enum
from typing import cast

from datapizza.core.clients.models import ClientResponse

CACHE_SCHEMA_VERSION = 1


class CachePayloadType(str, Enum):
    CLIENT_RESPONSE = "client_response"
    JSON = "json"


def encode_cache_value(obj: object) -> bytes:
    if isinstance(obj, ClientResponse):
        response = cast(ClientResponse, obj)
        envelope = {
            "v": CACHE_SCHEMA_VERSION,
            "t": CachePayloadType.CLIENT_RESPONSE,
            "p": response.to_dict(),
        }
    else:
        envelope = {
            "v": CACHE_SCHEMA_VERSION,
            "t": CachePayloadType.JSON,
            "p": obj,
        }

    return json.dumps(envelope, separators=(",", ":")).encode("utf-8")


def decode_cache_value(raw_value: bytes | str) -> object | None:
    if isinstance(raw_value, str):
        raw_bytes = raw_value.encode("utf-8")
    elif isinstance(raw_value, bytes):
        raw_bytes = raw_value
    else:
        raise ValueError(f"unsupported redis payload type: {type(raw_value)!r}")

    envelope = json.loads(raw_bytes.decode("utf-8"))
    if not isinstance(envelope, dict):
        raise ValueError("cache envelope must be a dict")

    if envelope.get("v") != CACHE_SCHEMA_VERSION:
        return None

    payload_type = envelope.get("t")
    payload = envelope.get("p")

    if payload_type == CachePayloadType.CLIENT_RESPONSE:
        if not isinstance(payload, dict):
            return None
        return ClientResponse.from_dict(payload)

    if payload_type == CachePayloadType.JSON:
        return payload

    return None



import logging

from datapizza.core.cache import Cache

import redis

from .json_codec import decode_cache_value, encode_cache_value

log = logging.getLogger(__name__)


class RedisCache(Cache):
    """
    A Redis-based cache implementation.
    """

    def __init__(
        self, host="localhost", port=6379, db=0, expiration_time=3600
    ):  # 1 hour default
        self.redis = redis.Redis(host=host, port=port, db=db)
        self.expiration_time = expiration_time

    def get(self, key: str) -> object | None:
        """Retrieve and deserialize object"""
        raw_value = self.redis.get(key)
        if raw_value is None:
            return None

        try:
            return decode_cache_value(raw_value)
        except Exception as exc:
            log.warning("Invalid cache payload for key %s: %s", key, exc)
            return None

    def set(self, key: str, obj: object):
        """Serialize and store object"""
        serialized = encode_cache_value(obj)
        self.redis.set(key, serialized, ex=self.expiration_time)
